﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using XamGrid_sample.Data;

namespace XamGrid_sample.ViewModel
{
    public class ViewModel : INotifyPropertyChanged
    {

        private ObservableCollection<OrderWrap> orders;
        public ObservableCollection<OrderWrap> Orders
        {
            get
            {
                if (orders == null)
                    orders = new ObservableCollection<OrderWrap>();
                return orders;
            }
            set
            {
                orders = value;
                NotifyPropertyChanged("Orders");
            }
        }

        private bool isAddNewRowVisible;
        public bool IsAddNewRowVisible
        {
            get { return isAddNewRowVisible; }
            set
            {
                isAddNewRowVisible = value;
                NotifyPropertyChanged("IsAddNewRowVisible");
            }
        }


        public ViewModel()
        {
            Order order1 = new Order() { Name = "O1" };
            Order order2 = new Order() { Name = "O2" };
            Order order3 = new Order() { Name = "O3" };
            Order order4 = new Order() { Name = "O4" };

            Item t1 = new Item() { Name = "Laptop" };
            Item t2 = new Item() { Name = "Table" }; 
            Item t3 = new Item() { Name = "TV" }; 
            Item t4 = new Item() { Name = "Smartphone" };

            order1.Items.Add(t1);
            order2.Items.Add(t2);
            order3.Items.Add(t3);
            order4.Items.Add(t4);

            Orders.Add(new OrderWrap(order1));
            Orders.Add(new OrderWrap(order2));
            Orders.Add(new OrderWrap(order3));
            Orders.Add(new OrderWrap(order4));



            //Orders.Add(order1);
            //Orders.Add(order2);
            //Orders.Add(order3);
            //Orders.Add(order4);
          
        }

        #region relaycom
        public class RelayCommand : ICommand
        {
            private Action<object> execute;
            private Func<object, bool> canExecute;

            public event EventHandler CanExecuteChanged
            {
                add { CommandManager.RequerySuggested += value; }
                remove { CommandManager.RequerySuggested -= value; }
            }

            public RelayCommand(Action<object> execute, Func<object, bool> canExecute = null)
            {
                this.execute = execute;
                this.canExecute = canExecute;
            }

            public bool CanExecute(object parameter)
            {
                return this.canExecute == null || this.canExecute(parameter);
            }

            public void Execute(object parameter)
            {
                this.execute(parameter);
            }
        }
        #endregion

        #region Notify Property Changed

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(string propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion // Notify Property Changed 
    }

    public class OrderWrap
    {
        public Order Order { get; set; }
        public OrderWrap(){ }
        public OrderWrap(Order O)
        {
            this.Order = O;
            this.Items = new ObservableCollection<ItemWrap>(
                from i in Order.Items
                select new ItemWrap(i));
        }
        public string Name
        {
            get
            {
                return Order.Name;
            }
        }
        public ObservableCollection<ItemWrap> Items
        {
            get;
            set;
        }
    }
    
    public class ItemWrap:INotifyPropertyChanged
    {
        public Item Item { get; set; }
        public ItemWrap() { }
        public ItemWrap(Item I)
        {
            this.Item= I;
        }

        public string Name
        {
            get
            {
                return Item.Name;
            }
        }
        public int? TotQte
        {
            get
            {
                return this.Deliveries.Sum(Del => Del.Qty);
            }
        }

        private ObservableCollection<Delivery> _Deliveries = null;
        public ObservableCollection<Delivery> Deliveries
        {
            get
            {
                if (_Deliveries == null)
                {
                    _Deliveries = new ObservableCollection<Delivery>(from Dev in Item.Deliveries
                                                                     select Dev);
                    _Deliveries.CollectionChanged += Deliveries_CollectionChanged;
                }
                return _Deliveries;
            }
            set
            {
                _Deliveries = value;
            }
        }
        
        void Deliveries_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                foreach (object item in e.NewItems)
                {
                    Delivery del = item as Delivery;
                    del.PropertyChanged += del_PropertyChanged;
                    //Update on data
                }
            }
            else if (e.Action==System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
            {
                foreach (object item in e.OldItems)
                {
                    Delivery del = item as Delivery;
                    del.PropertyChanged -= del_PropertyChanged;
                    //Update on data
                }
            }

            NotifyPropertyChanged("TotQte");
        }

        void del_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            NotifyPropertyChanged("TotQte");
        }

        #region Notify Property Changed

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(string propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion // Notify Property Changed 
    }

    //public class DeliveryWrap
    //{
    //    //public Delivery Delivery = null;
    //    //public DeliveryWrap()
    //    //{
    //    //    if (Delivery == null)
    //    //        Delivery = new Delivery();
    //    //    Delivery.PropertyChanged += Delivery_PropertyChanged;
    //    //}
        
    //    //public DeliveryWrap(Delivery D)
    //    //{
    //    //    this.Delivery = D;
    //    //    Delivery.PropertyChanged += Delivery_PropertyChanged;
    //    //}

    //    //void Delivery_PropertyChanged(object sender, PropertyChangedEventArgs e)
    //    //{
    //    //    NotifyPropertyChanged("Deliveries");
    //    //}

    //    //public int? Qty
    //    //{
    //    //    get
    //    //    {
    //    //        if (Delivery == null)
    //    //            return null;
    //    //        return Delivery.Qty;
    //    //    }
    //    //    set
    //    //    {
    //    //        Delivery.Qty = value;
    //    //    }
    //    //}


    //    #region Notify Property Changed

    //    public event PropertyChangedEventHandler PropertyChanged;
    //    private void NotifyPropertyChanged(string propertyName = "")
    //    {
    //        if (PropertyChanged != null)
    //        {
    //            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    //        }
    //    }

    //    #endregion // Notify Property Changed
    //}
}
